%   KOoscanal.m
%
%       Analyzes oscillating knollenorgan responses.
%
%           Takes as input:
%             response = recorded response vector
%             srate_rec = recording sampling rate 
%             prestimdur = prestimulus duration
%             pulsetype = stimulus pulse type 
%             pulsedur = stimulus pulse duration
%             pulseint = stimulus interpulse interval
%           Returns as output:
%             firstpd_ofmean = first poststimulus period of the mean response across sweeps, measured as the difference in time between the first and second poststimulus peaks
%             firstpktime_ofmean = time of the first poststimulus peak of the mean response across sweeps
%             firstpd_mean = mean of the first poststimulus period, measured on each sweep then averaged across sweeps
%             firstpd_sd = standard deviation of the first poststimulus period across sweeps
%             firstpktime_mean = mean of the time of the first poststimulu peak, measured on each sweep then averaged across sweeps
%             firstpktime_sd = standard deviation of the first poststimulus peak across sweep
%             vector_str = vector strength of the phase of oscillation across sweeps
%             phase_mean = average of the poststimulus oscillation phase across sweeps
%             phase_sd = standard deviation of the poststimulus oscillation phase across sweeps
%             pdofmean_prob = 0 if correct peaks measured on mean response, 1 if incorrect peaks measured
%             mean_p2pamp = mean first poststimulus peak to peak oscillation amplitude 
%             std_p2pamp = standard deviation of the first poststimulus oscillation amplitude
%             mean_p2pamp_prestim = mean prestimulus peak to peak oscillation amplitude
%             std_p2pamp_prestim = standard deviation of the prestimulus oscillation amplitude
%             miscounted_reps = 0 if first 7 peaks measured correctly on all reps, 1 if first 7 peaks measured incorrected on any rep
%             missedfirstpeak = 0 if first peak correctly measusred on all reps, 1 if first peak missed on any rep
%             first_artintrus = 0 if first peak not affected by artifact, 1 if first peak affected by artifact on any rep
%             analysis_type = 'A' if file analyzed peaks automatically, 'M' if peaks were manually chosen on any rep 
%             phase = phase of first poststimulus oscillation
%             z = z-value of Rayleigh test on first poststimulus oscillatory phase
%             pval = p-value of Raleigh test on first poststimulus oscillatory phase
%             delta_t_mean = not applicable for single pulse stimuli
%             delta_t_sd =  not applicable for single pulse stimuli
%             mean_first_peak_when = not applicable for single pulse stimuli
%             mean_first_peak_when_first = not applicable for single pulse stimuli
% function [firstpd_ofmean firstpklat_ofmean firstpd_mean firstpd_sd firstpklat_mean firstpklat_sd vector_str phase_mean phase_sd pdofmean_prob mean_p2pamp std_p2pamp miscounted_reps missedfirstpeak first_artintrus analysis_type]=KOoscanal(response, srate_rec, prestimdur, pulsetype, pulsedur, pulseint, wavfilename, stimulus, numpeaks)
% function [firstpd_ofmean firstpklat_ofmean firstpd_mean firstpd_sd firstpklat_mean firstpklat_sd vector_str phase_mean phase_sd pdofmean_prob mean_p2pamp std_p2pamp miscounted_reps missedfirstpeak first_artintrus analysis_type phase z pval]=KOoscanal(response, srate_rec, prestimdur, pulsetype, pulsedur, pulseint, wavfilename, stimulus, numpeaks) 
function [firstpd_ofmean firstpklat_ofmean firstpd_mean firstpd_sd firstpklat_mean firstpklat_sd vector_str phase_mean phase_sd pdofmean_prob mean_p2pamp std_p2pamp mean_p2pamp_prestim std_p2pamp_prestim miscounted_reps missedfirstpeak first_artintrus analysis_type phase z pval delta_t_mean delta_t_sd mean_first_peak_when mean_first_peak_when_first]=KOoscanal(response, srate_rec, prestimdur, pulsetype, pulsedur, pulseint, wavfilename, stimulus, numpeaks)


%% First cell: for troubleshooting KOoscanal only
% Load file
% close all
% clear all 
% 
% [filename, pathname] = uigetfile('*.wave.KOrec.mat', 'Open file');
% load (fullfile(pathname,filename), '-mat');
% cd(char(pathname))
% numpeaks=7;
%% Set some initial parameters
time_rec=[1:1:length(response)].*(1000/srate_rec);
miscounted_reps=0;
missedfirstpeak=0;
first_artintrus=0;
pdofmean_prob=0;
pad=0;
analysis_type='A'; %default to 'A'utomatic analysis
stimtype=pulsetype(1);
[nreps b]=size(response);
period_probs=zeros(1,nreps);
peak_time_probs=zeros(1,nreps);
peak_artifact_probs=zeros(1,nreps);
artintrus=zeros(1,nreps);
to_mananal=zeros(1,nreps);
to_mananal_first=zeros(1,nreps);
for i=1:nreps    
filt_response(i,:)=medfilt1(response(i,:),10,50); %10 points=0.10 ms
end
mean_response=mean(filt_response); 
%% For natural EOD stimuli, measure stimulus duration
if (stimtype=='L')
    wavfilename=wavfilename(2:end);
    wavfilename=strcat('Y',wavfilename);
    load (wavfilename);
    time_stm=[1:length(stimwave)].*(1000/srate);
    stimamp=max(stimwave)-min(stimwave);
    %Define EOD start and end using 2% of peak to peak amplitude criterion.
    %Use absolute value of mean stimulus to accomodate negative and
    %positive phases.
EODstart_stm=find(abs(stimwave)>(0.02*stimamp),1)-1;
EODend_stm=find(abs(stimwave)>(0.02*stimamp),1,'last')+1;
EODstart_stmtime=time_stm(EODstart_stm);
EODend_stmtime=time_stm(EODend_stm);
if (isempty(pulsedur))
    pulsedur=EODend_stmtime-EODstart_stmtime;
end
pad=EODstart_stmtime;
[q r]=size(stimulus);
figure
hold on
for i=1:q
    plot(time_rec,stimulus(i,:),'k')
end
plot([EODstart_stmtime+prestimdur EODstart_stmtime+prestimdur],[min(stimulus(1,:)) max(stimulus(1,:))],'g')
plot([EODend_stmtime+prestimdur EODend_stmtime+prestimdur],[min(stimulus(1,:)) max(stimulus(1,:))],'r')
xlabel('Time (ms)')
ylabel('Stimulus amplitude')
title('Stimulus: black=stimuli, green=start, red=end')
end
%% Measure only responses to second pulse if paired pulse stimuli
if (~isempty(pulseint))
    pad=pulseint;
%     pad=0;
end

%% Measure first poststimulus peaks: each sweep
for i=1:nreps
    prestimamps(i)=max(filt_response(i,1:round(prestimdur/1000*srate_rec)))-min(filt_response(i,1:round(prestimdur/1000*srate_rec)));
end
mean_prestimamp_byrep=mean(prestimamps);
% wins=prestimdur+pad+0.2;
wins=prestimdur+pad;
if (wins<(prestimdur+pad+pulsedur))
    wins=prestimdur+pad+pulsedur;
end
for i=1:nreps
[allpeaks alltroughs]=peakdet(filt_response(i,:),0.025*mean_prestimamp_byrep);
peak_times=time_rec(allpeaks(:,1));
trough_times=time_rec(alltroughs(:,1));
first_ind=find(peak_times>wins,1);
peak_inds(1)=first_ind;
for j=1:numpeaks-1
   peak_inds(j+1)=peak_inds(j)+1;
end
for j=1:length(peak_inds)
    k=allpeaks(peak_inds(j),:);
    peaks_where(i,j)=k(1);
    peak_amps(i,j)=k(2);
    peaks_when(i,j)=time_rec(peaks_where(i,j));
    clear k
end
for j=1:length(peak_inds)-1
    periods(i,j)=peaks_when(i,j+1)-peaks_when(i,j);
end
first_period(i)=peaks_when(i,2)-peaks_when(i,1);
first_peak_when(i)=peaks_when(i,1);
first_peak_amp(i)=peak_amps(i,1);
sec_peak_when(i)=peaks_when(i,2);
sec_peak_amp(i)=peak_amps(i,2);
if (numpeaks>2)
mean_periods_last5(i)=mean(periods(i,2:end));
std_periods_last5(i)=std(periods(i,2:end));
cv_periods_last5(i)=std_periods_last5(i)/mean_periods_last5(i);
end
end
%% Check 1: coefficient of variation of 2nd-6th periods, OR 2nd peak occurs later than 2 periods after stimulus offset, OR 1st peak occurs later than 1 period after stimulus offset 
%       If CV too high, ask user whether program is missing peaks. If yes, allow user to manually pick peaks.
%       OR
%       If 2nd peak too late, ask user whether program is missing peaks. If so, allow user to manually pick peaks.
%       OR
%       If 1st peak too late, ask user whether program is missing peaks. If
%       so, allow user to manually pick peaks.
if (numpeaks>2)
[c d]=size(periods);
for i=1:nreps
    if ((cv_periods_last5(i)>0.1) || (peaks_when(i,2)>(prestimdur+pulsedur+2.3*mean_periods_last5(i))) || (peaks_when(i,1)>(prestimdur+pulsedur+1.15*mean_periods_last5(i))))
        if (cv_periods_last5(i)>0.1)
            disp([' '])
            disp('Flag: CV of 2nd-6th periods is high')
        elseif (peaks_when(i,2)>(prestimdur+pulsedur+2.3*mean_periods_last5(i)))
            disp([' '])
                disp('Flag: 2nd peak occurs later than 2 periods after stimulus onset')
        elseif (peaks_when(i,1)>(prestimdur+pulsedur+1.15*mean_periods_last5(i)))
            disp([' '])
            disp('Flag: 1st peak occurs later than 1 period after stimulus onset')
        end
        figure
        plot(time_rec,response(i,:),'k')
        hold on
        plot(time_rec,filt_response(i,:),'m')
        plot(peaks_when(i,:),peak_amps(i,:),'c*')
        xlabel('Time (ms)')
        ylabel('Resp Amp (mV)')
        title(num2str(i))
        legend('Recorded response','Filtered response')
        %break out of automation and ask user if peaks are miscounted (ie,
        %if covariance is high but peaks are real, program will do nothing)
        disp([' '])
        miscount=input('Miscounting peaks? (y/n)','s');
        if ((miscount~='y') && (miscount~='n'))
            ask=1;
            while (ask==1)
            miscount=input('y/n required! Miscounting peaks? (y/n)','s');
            if ((miscount=='y')||(miscount=='n'))
                ask=0;
            end
            end
        end
           if (miscount=='y')
            miscounted_reps=1;
            period_probs(i)=1;
            to_mananal(i)=1;
        end
    end
end
end

%% Ask user whether to manually pick peaks (if numpeaks==2)
if (numpeaks==2)
    pickpeaks=input('Manually pick two poststimulus peaks? (y/n)','s');
if (pickpeaks=='y')
    reps_torecount=1:nreps;
    for j=1:length(reps_torecount)
        [peaks_where(reps_torecount(j),:) peak_amps(reps_torecount(j),:)]=KOmananal(response(reps_torecount(j),:),mean_response,time_rec,reps_torecount(j),prestimdur,pulsedur,pulseint,numpeaks);
        peaks_when(reps_torecount(j),:)=time_rec(peaks_where(reps_torecount(j),:));
        analysis_type='M'; %change analysis type to 'M'anual
    end
elseif (pickpeaks=='n')
    reps_torecount=[];
end
end

%% Prompt user to manually pick peaks if necessary
if (numpeaks>2)
reps_torecount=find(to_mananal==1);
if ~isempty(reps_torecount)
for j=1:length(reps_torecount)
    [peaks_where(reps_torecount(j),:) peak_amps(reps_torecount(j),:)]=KOmananal(response(reps_torecount(j),:),mean_response,time_rec,reps_torecount(j),prestimdur,pulsedur,pulseint,numpeaks);
    peaks_when(reps_torecount(j),:)=time_rec(peaks_where(reps_torecount(j),:));
    analysis_type='M'; %change analysis type to 'M'anual
end
end
end

%% Adjust relevant parameters
if ~isempty(reps_torecount)
    [c d]=size(peaks_when);
    for i=1:nreps
        for j=1:d-1
            periods(i,j)=peaks_when(i,j+1)-peaks_when(i,j);
        end
        first_period(i)=peaks_when(i,2)-peaks_when(i,1);
        first_peak_when(i)=peaks_when(i,1);
        first_peak_amp(i)=peak_amps(i,1);
        sec_peak_when(i)=peaks_when(i,2);
        sec_peak_amp(i)=peak_amps(i,2);
        if (numpeaks>2)
        mean_periods_last5(i)=mean(periods(i,2:end));
        std_periods_last5(i)=std(periods(i,2:end));
        cv_periods_last5(i)=std_periods_last5(i)/mean_periods_last5(i);
        mean_periods_all6(i)=mean(periods(i,:));
        std_periods_all6(i)=std(periods(i,:));
        cv_periods_all6(i)=std_periods_all6(i)/mean_periods_all6(i);
        end
    end
end

%% Check for artifact intrusion into 1st peak
    %If first period is <85% or >115% of mean of 2nd-6th periods, throw out first peak.
    if (numpeaks>2)
    for i=1:nreps
        if (first_period(i)>1.15*mean_periods_last5(i) || (first_period(i)<0.85*mean_periods_last5(i)))
            artintrus(i)=1;
        end
    end
    if (sum(artintrus>0))
        to_discard=find(artintrus==1);
        figure
        hold on
        for i=1:nreps
            plot(time_rec,response(i,:),'k')
            plot(first_peak_when(i),first_peak_amp(i),'c*')
            plot(sec_peak_when(i),sec_peak_amp(i),'b*')
            plot(first_peak_when(to_discard),first_peak_amp(to_discard),'r*')
        end
        xlabel('Time (ms)')
        ylabel('Resp Amp (mV)')
        title('All sweeps: throw out first peaks?')
        legend('Recorded response','First peaks','Second peaks','First peaks affected by artifact?')
        if (sum(artintrus>0))
        disp([' '])
        discard_artintrus=input('Throw out first peaks affected by artifact? (y/n)','s');
        if ((discard_artintrus~='y')&&(discard_artintrus~='n'))
            ask=1;
            while (ask==1)
                discard_artintrus=input('y/n required! Throw out first peaks affected by artifact? (y/n)','s');
                if (discard_artintrus=='n')
                    ask=0;
                elseif (discard_artintrus=='y')
                    ask=0;
                end
            end
        end
        if (discard_artintrus=='y')
            to_discard=find(artintrus==1);
            for j=1:length(to_discard)
            first_artintrus=1;
            first_period(to_discard(j))=periods(to_discard(j),2);
            first_peak_when(to_discard(j))=peaks_when(to_discard(j),2);
            first_peak_amp(to_discard(j))=peak_amps(to_discard(j),2);
            sec_peak_when(to_discard(j))=peaks_when(to_discard(j),3);
            sec_peak_amp(to_discard(j))=peak_amps(to_discard(j),3);
            end
              figure
            for i=1:nreps
        plot(time_rec,response(i,:),'k')
        hold on
        plot(first_peak_when(i),first_peak_amp(i),'c*')
        plot(sec_peak_when(i),sec_peak_amp(i),'b*')
        xlabel('Time (ms)')
        ylabel('Resp Amp (mV)')
        t=strcat('First peak discarded due to artifact effects: sweep ',num2str(i));
        title(t)
        legend('Recorded response','First peak','Second peak')
        end
        end
    end
    end
    end
    
     figure
        hold on
        for i=1:nreps
            plot(time_rec,response(i,:),'k')
            plot(first_peak_when(i),first_peak_amp(i),'c*')
            plot(sec_peak_when(i),sec_peak_amp(i),'b*')
        end
        xlabel('Time (ms)')
        ylabel('Resp Amp (mV)')
        title('All sweeps: throw out first peaks?')
        legend('Recorded response','First peaks','Second peaks')

%% Ask user whether to "remove" any first peaks 
remove=input('Remove any first peaks? (y/n)','s');
if ((remove~='y') && (remove~='n'))
            ask=1;
            while (ask==1)
            remove=input('y/n required! Remove any first peaks? (y/n)','s');
            if ((remove=='y')||(remove=='n'))
                ask=0;
            end
            end
end
if (remove=='y')
    ws=input('Set window START for removing first peaks (ms)');
    we=input('Set window END for removing first peaks (ms)');
    for i=1:nreps
        if((first_peak_when(i)>ws) && (first_peak_when(i)<we))
              first_period(i)=periods(i,2);
            first_peak_when(i)=peaks_when(i,2);
            first_peak_amp(i)=peak_amps(i,2);
            sec_peak_when(i)=peaks_when(i,3);
            sec_peak_amp(i)=peak_amps(i,3);
        end
    end  
    for i=1:nreps
        if((first_peak_when(i)>ws) && (first_peak_when(i)<we))
            'here'
              first_period(i)=periods(i,3);
            first_peak_when(i)=peaks_when(i,3);
            first_peak_amp(i)=peak_amps(i,3);
            sec_peak_when(i)=peaks_when(i,4);
            sec_peak_amp(i)=peak_amps(i,4);
        end
    end  
%     figure
    hold on
        for i=1:nreps
            figure
            hold on
            plot(time_rec,response(i,:),'k')
            plot(first_peak_when(i),first_peak_amp(i),'c*')
            plot(sec_peak_when(i),sec_peak_amp(i),'b*')
            title(i)
        end
        xlabel('Time (ms)')
        ylabel('Resp Amp (mV)')
        title('All sweeps: Identified first peaks removed')
        legend('Recorded response','First peaks','Second peaks')
end

%% Calculate relevant parameters
for i=1:nreps
    first_peak_lat(i)=first_peak_when(i)-prestimdur-pulsedur-pad;
    phase(i)=mod360(first_peak_lat(i)/first_period(i)*360);
    [t troughstart]=min(abs(time_rec-first_peak_when(i)));
    [e troughend]=min(abs(time_rec-sec_peak_when(i)));
    [troughamp(i) troughind(i)]=min(response(i,troughstart:troughend));
    troughwhen(i)=time_rec(troughind(i)+troughstart);
    p2pamp(i)=mean([first_peak_amp(i) sec_peak_amp(i)])-troughamp(i);
end
mean_first_peak_when=mean(first_peak_when);
mean_latency=mean(first_peak_lat);
sd_latency=std(first_peak_lat);
mean_first_period=mean(first_period);
sd_period=std(first_period);
mean_firstamp=mean(first_peak_amp);
mean_secamp=mean(sec_peak_amp);
mean_p2pamp=mean(p2pamp)
std_p2pamp=std(p2pamp);
[vec_str,mean_angle,std_angle,sem_angle] = vs(phase);
[r,mean_angle2,z,S,pval] = rayleigh(phase)
% [vec_str_first,mean_angle_first,std_angle_first,sem_angle_first] = vs(phase_first);
% [r_first,mean_angle2_first,z_first,S_first,pval_first] = rayleigh(phase_first);
figure
    hold on
        for i=1:nreps
            plot(time_rec,response(i,:),'k')
            plot(first_peak_when(i),first_peak_amp(i),'c*')
            plot(sec_peak_when(i),sec_peak_amp(i),'b*')
            plot(troughwhen(i),troughamp(i),'g*')
        end
        xlabel('Time (ms)')
        ylabel('Resp Amp (mV)')
        title('All sweeps: Identified first peaks removed')
        legend('Recorded response','First peaks','Second peaks','Troughs')
        
%% Measure prestimulus oscillation amplitudes
clear allpeaks alltroughs
for i=1:nreps   
[allpeaks alltroughs]=peakdet(filt_response(i,1:(round(prestimdur/1000*srate_rec)-1)),0.025*mean_prestimamp_byrep);
allpeaks=allpeaks(2:end,:);
[pa pb]=size(allpeaks);
[ta tb]=size(alltroughs);
    if (allpeaks(1,1)<alltroughs(1,1))
        for j=1:(min([pa ta])-1)
            amps(j)=allpeaks(j,2)-alltroughs(j+1,2);
        end
    elseif (allpeaks(1,1)>alltroughs(1,1))
        for j=1:(min([pa ta])-1)
            amps(j)=allpeaks(j+1,2)-alltroughs(j,2);
        end
    end
    mean_amps(i)=mean(amps);
   
    figure
    plot(time_rec,response(i,:),'k')
%     plot(time_rec,stimulus(i,:),'k')
    hold on
    plot(time_rec,filt_response(i,:),'m')
    scatter(time_rec(allpeaks(:,1)),allpeaks(:,2),'b*')
    scatter(time_rec(alltroughs(:,1)),alltroughs(:,2),'g*')
    xlabel('Time (ms)')
    ylabel('Voltage (mV)')
    title(i)
    clear amps allpeaks alltroughs 
end
mean_p2pamp_prestim=mean(mean_amps);
std_p2pamp_prestim=std(mean_amps);
%% Measure time of peaks and period of average response

mean_prestimamp=max(mean_response(1:round(prestimdur/1000*srate_rec)))-min(mean_response(1:round(prestimdur/1000*srate_rec)));
wins=prestimdur+pad+0.2;
if (wins<(prestimdur+pulsedur+pad))
    wins=prestimdur+pulsedur+pad;
end
[allpeaks alltroughs]=peakdet(mean_response,0.025*mean_prestimamp);
peak_times=time_rec(allpeaks(:,1));
trough_times=time_rec(alltroughs(:,1));
first_ind=find(peak_times>wins,1);
sec_ind=first_ind+1;
third_ind=first_ind+2;
h=allpeaks(first_ind,:);
j=allpeaks(sec_ind,:);
k=allpeaks(third_ind,:);
mfirst_peakwhere=h(1);
mfirst_amp=h(2);
mfirst_peakwhen=time_rec(mfirst_peakwhere);
mfirst_peaklat=mfirst_peakwhen-prestimdur-pulsedur-pad;
msec_peakwhere=j(1);
msec_amp=j(2);
msec_peakwhen=time_rec(msec_peakwhere);
mthird_peakwhere=k(1);
mthird_amp=k(2);
mthird_peakwhen=time_rec(mthird_peakwhere);
mfirst_period=msec_peakwhen-mfirst_peakwhen;
% 
figure
hold on
for i=1:nreps
    plot(time_rec,response(i,:),'k')
    plot(time_rec,mean_response,'r','LineWidth',2)
    plot(mfirst_peakwhen,mfirst_amp,'c*')
    plot(msec_peakwhen,msec_amp,'b*')
end
xlabel('Time (ms)')
ylabel('Resp Amp (mV)')
title('Averaged response')
xlim([prestimdur-2 wins+7])
% 
%Ask user if correct peaks measured
correctper=input('Correct peaks measured on averaged response? (y/n)','s');
if ((correctper~='y') && (correctper~='n'))
    ask=1;
    while (ask==1)
        correctper=input('y/n required! Correct peaks measured on averaged response? (y/n)','s');
        if ((correctper=='y')||(correctper=='n'))
            ask=0;
        end
    end
end
if (correctper=='n')
    pdofmean_prob=1;
    plot(msec_peakwhen,msec_amp,'g*')
    plot(mthird_peakwhen,mthird_amp,'g*')
    better=input('New peaks (green) better? (y/n)','s');
    if ((better~='y')&&(better~='n'))
        ask=1;
        while (ask==1)
            better=input('y/n required! New peaks (green) better? (y/n)','s');
            if ((better=='y')||(better=='n'))
                ask=0;
            end
        end
    end
    if (better=='n')
        mfirst_period=0;
        mfirst_peaklat=0;
    elseif (better=='y')
        mfirst_period=mthird_peakwhen-msec_peakwhen;
        mfirst_peaklat=msec_peakwhen-prestimdur-pulsedur-pad;
    end
end
% 
firstpd_ofmean = mfirst_period;
firstpklat_ofmean = mfirst_peaklat;
firstpd_mean = mean_first_period;
firstpd_sd = sd_period;
firstpklat_mean = mean_latency;
firstpklat_sd = sd_latency;
vector_str = vec_str;
phase_mean = mean_angle;
phase_sd = std_angle;
% %% Measure responses to first pulse in pair
% % if (~isempty(pulseint))
%     pad=0;
% %% Set some initial parameters
% period_probs=zeros(1,nreps);
% peak_time_probs=zeros(1,nreps);
% peak_artifact_probs=zeros(1,nreps);
% artintrus=zeros(1,nreps);
% to_mananal=zeros(1,nreps);
% clear to_discard
% %% Measure first poststimulus peaks: each sweep
% 
% % wins=prestimdur+pad+0.2;
% wins=prestimdur;
% % if (wins<(prestimdur+pad+pulsedur))
% %     wins=prestimdur+pad+pulsedur;
% % end
% for i=1:nreps
% [allpeaks alltroughs]=peakdet(filt_response(i,:),0.025*mean_prestimamp_byrep);
% peak_times=time_rec(allpeaks(:,1));
% trough_times=time_rec(alltroughs(:,1));
% first_ind_first=find(peak_times>wins,1);
% peak_inds_first(1)=first_ind_first;
% for j=1:numpeaks-1
%    peak_inds_first(j+1)=peak_inds_first(j)+1;
% end
% for j=1:length(peak_inds_first)
%     k=allpeaks(peak_inds_first(j),:);
%     peaks_where_first(i,j)=k(1);
%     peak_amps_first(i,j)=k(2);
%     peaks_when_first(i,j)=time_rec(peaks_where_first(i,j));
%     clear k
% end
% for j=1:length(peak_inds_first)-1
%     periods_first(i,j)=peaks_when_first(i,j+1)-peaks_when_first(i,j);
% end
% first_period_first(i)=peaks_when_first(i,2)-peaks_when_first(i,1);
% first_peak_when_first(i)=peaks_when_first(i,1);
% first_peak_amp_first(i)=peak_amps_first(i,1);
% sec_peak_when_first(i)=peaks_when_first(i,2);
% sec_peak_amp_first(i)=peak_amps_first(i,2);
% if (numpeaks>2)
% mean_periods_last5_first(i)=mean(periods_first(i,2:end));
% std_periods_last5_first(i)=std(periods_first(i,2:end));
% cv_periods_last5_first(i)=std_periods_last5_first(i)/mean_periods_last5_first(i);
% end
% end
% 
% %% Check 1: coefficient of variation of 2nd-6th periods, OR 2nd peak occurs later than 2 periods after stimulus offset, OR 1st peak occurs later than 1 period after stimulus offset 
% %       If CV too high, ask user whether program is missing peaks. If yes, allow user to manually pick peaks.
% %       OR
% %       If 2nd peak too late, ask user whether program is missing peaks. If so, allow user to manually pick peaks.
% %       OR
% %       If 1st peak too late, ask user whether program is missing peaks. If
% %       so, allow user to manually pick peaks.
% if (numpeaks>2)
% [c d]=size(periods_first);
% for i=1:nreps
%     if ((cv_periods_last5_first(i)>0.1) || (peaks_when_first(i,2)>(prestimdur+pulsedur+2.3*mean_periods_last5_first(i))) || (peaks_when_first(i,1)>(prestimdur+pulsedur+1.15*mean_periods_last5_first(i))))
%         if (cv_periods_last5_first(i)>0.1)
%             disp([' '])
%             disp('Flag: CV of 2nd-6th periods is high')
%         elseif (peaks_when_first(i,2)>(prestimdur+pulsedur+2.3*mean_periods_last5_first(i)))
%             disp([' '])
%                 disp('Flag: 2nd peak occurs later than 2 periods after stimulus onset')
%         elseif (peaks_when_first(i,1)>(prestimdur+pulsedur+1.15*mean_periods_last5_first(i)))
%             disp([' '])
%             disp('Flag: 1st peak occurs later than 1 period after stimulus onset')
%         end
%         figure
%         plot(time_rec,response(i,:),'k')
%         hold on
%         plot(time_rec,filt_response(i,:),'m')
%         plot(peaks_when_first(i,:),peak_amps_first(i,:),'c*')
%         xlabel('Time (ms)')
%         ylabel('Resp Amp (mV)')
%         title(num2str(i))
%         legend('Recorded response','Filtered response')
%         %break out of automation and ask user if peaks are miscounted (ie,
%         %if covariance is high but peaks are real, program will do nothing)
%         disp([' '])
%         miscount=input('Miscounting peaks? (y/n)','s');
%         if ((miscount~='y') && (miscount~='n'))
%             ask=1;
%             while (ask==1)
%             miscount=input('y/n required! Miscounting peaks? (y/n)','s');
%             if ((miscount=='y')||(miscount=='n'))
%                 ask=0;
%             end
%             end
%         end
%            if (miscount=='y')
%             miscounted_reps=1;
%             period_probs(i)=1;
%             to_mananal(i)=1;
%         end
%     end
% end
% end
% 
% %% Ask user whether to manually pick peaks (if numpeaks==2)
% clear pickpeaks
% if (numpeaks==2)
%     pickpeaks=input('Manually pick two poststimulus peaks? (y/n)','s');
% if (pickpeaks=='y')
%     reps_torecount=1:nreps;
%     for j=1:length(reps_torecount)
%         [peaks_where_first(reps_torecount(j),:) peak_amps_first(reps_torecount(j),:)]=KOmananal(response(reps_torecount(j),:),mean_response,time_rec,reps_torecount(j),prestimdur,pulsedur,pulseint,numpeaks);
%         peaks_when_first(reps_torecount(j),:)=time_rec(peaks_where_first(reps_torecount(j),:));
%         analysis_type='M'; %change analysis type to 'M'anual
%     end
% elseif (pickpeaks=='n')
%     reps_torecount=[];
% end
% end
% 
% %% Prompt user to manually pick peaks if necessary
% if (numpeaks>2)
% reps_torecount=find(to_mananal==1);
% if ~isempty(reps_torecount)
% for j=1:length(reps_torecount)
%     [peaks_where_first(reps_torecount(j),:) peak_amps_first(reps_torecount(j),:)]=KOmananal(response(reps_torecount(j),:),mean_response,time_rec,reps_torecount(j),prestimdur,pulsedur,pulseint,numpeaks);
%     peaks_when_first(reps_torecount(j),:)=time_rec(peaks_where_first(reps_torecount(j),:));
%     analysis_type='M'; %change analysis type to 'M'anual
% end
% end
% end
% 
% %% Adjust relevant parameters
% if ~isempty(reps_torecount)
%     [c d]=size(peaks_when_first);
%     for i=1:nreps
%         for j=1:d-1
%             periods_first(i,j)=peaks_when_first(i,j+1)-peaks_when_first(i,j);
%         end
%         first_period_first(i)=peaks_when_first(i,2)-peaks_when_first(i,1);
%         first_peak_when_first(i)=peaks_when_first(i,1);
%         first_peak_amp_first(i)=peak_amps_first(i,1);
%         sec_peak_when_first(i)=peaks_when_first(i,2);
%         sec_peak_amp_first(i)=peak_amps_first(i,2);
%         if (numpeaks>2)
%         mean_periods_last5_first(i)=mean(periods_first(i,2:end));
%         std_periods_last5_first(i)=std(periods_first(i,2:end));
%         cv_periods_last5_first(i)=std_periods_last5_first(i)/mean_periods_last5_first(i);
%         mean_periods_all6_first(i)=mean(periods_first(i,:));
%         std_periods_all6_first(i)=std(periods_first(i,:));
%         cv_periods_all6_first(i)=std_periods_all6_first(i)/mean_periods_all6_first(i);
%         end
%     end
% end
% 
% %% Check for artifact intrusion into 1st peak
%     %If first period is <85% or >115% of mean of 2nd-6th periods, throw out first peak.
%     if (numpeaks>2)
%     for i=1:nreps
%         if (first_period_first(i)>1.15*mean_periods_last5_first(i) || (first_period_first(i)<0.85*mean_periods_last5_first(i)))
%             artintrus(i)=1;
%         end
%     end
%     if (sum(artintrus>0))
%         to_discard=find(artintrus==1);
%         figure
%         hold on
%         for i=1:nreps
%             plot(time_rec,response(i,:),'k')
%             plot(first_peak_when_first(i),first_peak_amp_first(i),'c*')
%             plot(sec_peak_when_first(i),sec_peak_amp_first(i),'b*')
%             plot(first_peak_when_first(to_discard),first_peak_amp_first(to_discard),'r*')
%         end
%         xlabel('Time (ms)')
%         ylabel('Resp Amp (mV)')
%         title('All sweeps: throw out first peaks?')
%         legend('Recorded response','First peaks','Second peaks','First peaks affected by artifact?')
%         if (sum(artintrus>0))
%         disp([' '])
%         discard_artintrus=input('Throw out first peaks affected by artifact? (y/n)','s');
%         if ((discard_artintrus~='y')&&(discard_artintrus~='n'))
%             ask=1;
%             while (ask==1)
%                 discard_artintrus=input('y/n required! Throw out first peaks affected by artifact? (y/n)','s');
%                 if (discard_artintrus=='n')
%                     ask=0;
%                 elseif (discard_artintrus=='y')
%                     ask=0;
%                 end
%             end
%         end
%         if (discard_artintrus=='y')
%             to_discard=find(artintrus==1);
%             for j=1:length(to_discard)
%             first_artintrus=1;
%             first_period_first(to_discard(j))=periods_first(to_discard(j),2);
%             first_peak_when_first(to_discard(j))=peaks_when_first(to_discard(j),2);
%             first_peak_amp_first(to_discard(j))=peak_amps_first(to_discard(j),2);
%             sec_peak_when_first(to_discard(j))=peaks_when_first(to_discard(j),3);
%             sec_peak_amp_first(to_discard(j))=peak_amps_first(to_discard(j),3);
%             end
%               figure
%             for i=1:nreps
%         plot(time_rec,response(i,:),'k')
%         hold on
%         plot(first_peak_when_first(i),first_peak_amp_first(i),'c*')
%         plot(sec_peak_when_first(i),sec_peak_amp_first(i),'b*')
%         xlabel('Time (ms)')
%         ylabel('Resp Amp (mV)')
%         t=strcat('First peak discarded due to artifact effects: sweep ',num2str(i));
%         title(t)
%         legend('Recorded response','First peak','Second peak')
%         end
%         end
%     end
%     end
%     end
%     
%      figure
%         hold on
%         for i=1:nreps
%             plot(time_rec,response(i,:),'k')
%             plot(first_peak_when_first(i),first_peak_amp_first(i),'c*')
%             plot(sec_peak_when_first(i),sec_peak_amp_first(i),'b*')
%         end
%         xlabel('Time (ms)')
%         ylabel('Resp Amp (mV)')
%         title('All sweeps: throw out first peaks?')
%         legend('Recorded response','First peaks','Second peaks')
% 
% %% Ask user whether to "remove" any first peaks 
% remove=input('Remove any first peaks? (y/n)','s');
% if ((remove~='y') && (remove~='n'))
%             ask=1;
%             while (ask==1)
%             remove=input('y/n required! Remove any first peaks? (y/n)','s');
%             if ((remove=='y')||(remove=='n'))
%                 ask=0;
%             end
%             end
% end
% if (remove=='y')
%     ws=input('Set window START for removing first peaks (ms)');
%     we=input('Set window END for removing first peaks (ms)');
%     for i=1:nreps
%         if((first_peak_when_first(i)>ws) && (first_peak_when_first(i)<we))
%               first_period_first(i)=periods_first(i,2);
%             first_peak_when_first(i)=peaks_when_first(i,2);
%             first_peak_amp_first(i)=peak_amps_first(i,2);
%             sec_peak_when_first(i)=peaks_when_first(i,3);
%             sec_peak_amp_first(i)=peak_amps_first(i,3);
%         end
%     end  
%     for i=1:nreps
%         if((first_peak_when_first(i)>ws) && (first_peak_when_first(i)<we))
%             'here'
%               first_period_first(i)=periods_first(i,3);
%             first_peak_when_first(i)=peaks_when_first(i,3);
%             first_peak_amp_first(i)=peak_amps_first(i,3);
%             sec_peak_when_first(i)=peaks_when_first(i,4);
%             sec_peak_amp_first(i)=peak_amps_first(i,4);
%         end
%     end  
% %     figure
%     hold on
%         for i=1:nreps
%             figure
%             hold on
%             plot(time_rec,response(i,:),'k')
%             plot(first_peak_when_first(i),first_peak_amp_first(i),'c*')
%             plot(sec_peak_when_first(i),sec_peak_amp_first(i),'b*')
%             title(i)
%         end
%         xlabel('Time (ms)')
%         ylabel('Resp Amp (mV)')
%         title('All sweeps: Identified first peaks removed')
%         legend('Recorded response','First peaks','Second peaks')
% end
% 
% %% Calculate relevant parameters
% for i=1:nreps
%     first_peak_lat_first(i)=first_peak_when_first(i)-prestimdur-pulsedur-pad;
%     phase_first(i)=mod360(first_peak_lat_first(i)/first_period_first(i)*360);
%     [t_first troughstart_first]=min(abs(time_rec-first_peak_when_first(i)));
%     [e_first troughend_first]=min(abs(time_rec-sec_peak_when_first(i)));
%     [troughamp_first(i) troughind_first(i)]=min(response(i,troughstart_first:troughend_first));
%     troughwhen_first(i)=time_rec(troughind_first(i)+troughstart_first);
%     p2pamp_first(i)=mean([first_peak_amp_first(i) sec_peak_amp_first(i)])-troughamp_first(i);
% end
% mean_first_peak_when_first=mean(first_peak_when_first);
% mean_latency_first=mean(first_peak_lat_first);
% sd_latency_first=std(first_peak_lat_first);
% mean_first_period_first=mean(first_period_first);
% sd_period_first=std(first_period_first);
% mean_firstamp_first=mean(first_peak_amp_first);
% mean_secamp_first=mean(sec_peak_amp_first);
% mean_p2pamp_first=mean(p2pamp_first)
% std_p2pamp_first=std(p2pamp_first);
% [vec_str,mean_angle,std_angle,sem_angle] = vs(phase);
% [r,mean_angle2,z,S,pval] = rayleigh(phase)
% [vec_str_first,mean_angle_first,std_angle_first,sem_angle_first] = vs(phase_first);
% [r_first,mean_angle2_first,z_first,S_first,pval_first] = rayleigh(phase_first);
% figure
%     hold on
%         for i=1:nreps
%             i
%             plot(time_rec,response(i,:),'k')
%             plot(first_peak_when(i),first_peak_amp(i),'c*')
%             plot(sec_peak_when(i),sec_peak_amp(i),'b*')
%             plot(troughwhen(i),troughamp(i),'g*')
%             plot(first_peak_when_first(i),first_peak_amp_first(i),'c*')
%             plot(sec_peak_when_first(i),sec_peak_amp_first(i),'b*')
%             plot(troughwhen_first(i),troughamp_first(i),'g*')
%             first_peak_when_first(i)
%         end
%         xlabel('Time (ms)')
%         ylabel('Resp Amp (mV)')
%         title('All sweeps: Identified first peaks removed')
%         legend('Recorded response','First peaks','Second peaks','Troughs')
% %% Measure time of peaks and period of average response
% mean_prestimamp=max(mean_response(1:round(prestimdur/1000*srate_rec)))-min(mean_response(1:round(prestimdur/1000*srate_rec)));
% wins=prestimdur+pad+0.2;
% if (wins<(prestimdur+pulsedur+pad))
%     wins=prestimdur+pulsedur+pad;
% end
% [allpeaks alltroughs]=peakdet(mean_response,0.025*mean_prestimamp);
% peak_times=time_rec(allpeaks(:,1));
% trough_times=time_rec(alltroughs(:,1));
% first_ind_first=find(peak_times>wins,1);
% sec_ind_first=first_ind_first+1;
% third_ind_first=first_ind_first+2;
% h=allpeaks(first_ind_first,:);
% j=allpeaks(sec_ind_first,:);
% k=allpeaks(third_ind_first,:);
% mfirst_peakwhere_first=h(1);
% mfirst_amp_first=h(2);
% mfirst_peakwhen_first=time_rec(mfirst_peakwhere_first);
% mfirst_peaklat_first=mfirst_peakwhen_first-prestimdur-pulsedur-pad;
% msec_peakwhere_first=j(1);
% msec_amp_first=j(2);
% msec_peakwhen_first=time_rec(msec_peakwhere_first);
% mthird_peakwhere_first=k(1);
% mthird_amp_first=k(2);
% mthird_peakwhen_first=time_rec(mthird_peakwhere_first);
% mfirst_period_first=msec_peakwhen_first-mfirst_peakwhen_first;
% % 
% figure
% hold on
% for i=1:nreps
%     plot(time_rec,response(i,:),'k')
%     plot(time_rec,mean_response,'r','LineWidth',2)
%     plot(mfirst_peakwhen_first,mfirst_amp_first,'c*')
%     plot(msec_peakwhen_first,msec_amp_first,'b*')
% end
% xlabel('Time (ms)')
% ylabel('Resp Amp (mV)')
% title('Averaged response')
% xlim([prestimdur-2 wins+7])
% % 
% %Ask user if correct peaks measured
% correctper=input('Correct peaks measured on averaged response? (y/n)','s');
% if ((correctper~='y') && (correctper~='n'))
%     ask=1;
%     while (ask==1)
%         correctper=input('y/n required! Correct peaks measured on averaged response? (y/n)','s');
%         if ((correctper=='y')||(correctper=='n'))
%             ask=0;
%         end
%     end
% end
% if (correctper=='n')
%     pdofmean_prob=1;
%     plot(msec_peakwhen_first,msec_amp_first,'g*')
%     plot(mthird_peakwhen_first,mthird_amp_first,'g*')
%     better=input('New peaks (green) better? (y/n)','s');
%     if ((better~='y')&&(better~='n'))
%         ask=1;
%         while (ask==1)
%             better=input('y/n required! New peaks (green) better? (y/n)','s');
%             if ((better=='y')||(better=='n'))
%                 ask=0;
%             end
%         end
%     end
%     if (better=='n')
%         mfirst_period_first=0;
%         mfirst_peaklat_first=0;
%     elseif (better=='y')
%         mfirst_period_first=mthird_peakwhen_first-msec_peakwhen_first;
%         mfirst_peaklat_first=msec_peakwhen_first-prestimdur-pulsedur-pad_first;
%     end
% end
% % 
% firstpd_ofmean_first = mfirst_period;
% firstpklat_ofmean_first = mfirst_peaklat;
% firstpd_mean_first = mean_first_period;
% firstpd_sd_first = sd_period;
% firstpklat_mean_first = mean_latency;
% firstpklat_sd_first = sd_latency;
% vector_str_first = vec_str;
% phase_mean_first = mean_angle;
% phase_sd_first = std_angle;
% delta_t=first_peak_when-first_peak_when_first;
% delta_t_mean=mean(delta_t);
% delta_t_sd=std(delta_t);
% disp('First peak latency=')
% firstpklat_mean_first+pulsedur+pad
% % elseif (isempty(pulseint))
% %     delta_t_mean=NaN;
% %     delta_t_sd=NaN;
% % end
delta_t_mean=NaN;
delta_t_sd=NaN;
firstpd_ofmean_first=NaN;
firstpklat_ofmean_first=NaN;
firstpd_mean_first=NaN;
firstpd_sd_first=NaN;
firstpklat_mean_first=NaN;
firstpklat_sd_first=NaN;
vector_str_first=NaN;
phase_mean_first=NaN;
phase_sd_first=NaN;
mean_first_peak_when_first=NaN;