function [F,D,D2,T] = sdf(time,width,prnt,max_tim,bin_siz,rtrn,tau,phase,export)

%
%	sdf.m
%	spike density function using a gaussian convolution
%       SEE Dayan and Abbott 2001, pp.13, eqn. 1.11
%       SEE Szucs 2000
%   
%	USAGE:
%		[F,D,D2,T] = sdf(time,width)
%			OR
%		[F,D,D2,T] = sdf(time,width,prnt,max_tim,bin_siz,rtrn,tau,phase,export)
%
%	VARIABLES:
%		F = spike density function
%		D = spike density derivative
%       D2 = 2nd spike density derivative
%		T = binned spike times
%		time = vector of spike times (sec)
%		width = width of the gaussian distribution used (4 standard deviations)
%       max_tim = maximum sampling time (sec); default = last spike time
%       bin_siz = bin size for spike train and SDF sampling; default = 1 msec
%		prnt = 1 for graphing, 0 for no graphing; default = 1
%		rtrn = 1 for return plot, 0 for no return plot; default = 1
%		tau = delay to be used for return plot; default = 50 ms
%		phase = 1 for phase plot, 0 for no phase plot; default = 1
%		export = 1 for exporting data to text, 0 for no
%

%   default options
if (nargin==2),
   prnt = 0;
   max_tim = max(time);
   bin_siz = 1;
   rtrn = 0;
   tau = 50;
   phase = 0;
   cutoffs = 1;
   export = 0;
end

%   standard deviation of the gaussian
sd = width/4;

%   gaussian convolution with spike train
x=[-1000:1:1000];
y=exp(-0.5*((x./sd).^2));
y=y/(sqrt(2*pi)*sd);
y=y/(sum(y)/1000);
[T]=bin(time,bin_siz);
z_pad = ((ceil(max_tim)*1000)/bin_siz)-length(T);
T = [T zeros(1,z_pad)];
F = conv(T,y);
F = F(1001:length(F));
F = F(1:length(T));
D=diff(F);
D2=diff(D);
ind=[1:1:length(F)];
ind = ind/1000;

%   plot graphics
if (prnt==1),
   figure('Name','Spike Density Function',...
      'NumberTitle','off')
	subplot(2,1,1)
	plot(ind,T,'b-')
	xlim([0 max(ind)])
	ylim([-0.1 1.1])
	ylabel('Binned Spikes')
	zoom on
	subplot(2,1,2)
	plot(ind,F,'b-')
	hold on
	xlim([0 max(ind)])
    xlabel('Time (s)')
	ylabel('SDF[s^-^1]')
   zoom on
end

%   plot return map [SDF(t+tau) vs. SDF(t)]
if (rtrn==1),
   sdf1 = F(1:1:(length(F)-tau));
   sdf2 = F((tau+1):1:length(F));
   figure('Name','Return Map',...
      'NumberTitle','off');
   plot(sdf1,sdf2,'r.')
   xlabel('SDF(t) [s^-^1]')
   ylabel('SDF(t+tau) [s^-^1]')
   axis('equal')
   axis('square')
   zoom on
end

% plot phase plot [SDD vs. SDF]
if (phase==1),
    figure('Name','Phase Plot',...
      'NumberTitle','off');
    plot(F(2:length(F)),D,'r.')
    xlabel('SDF [s^-^1]')
    ylabel('SDD [s^-^2]')
    zoom on
end

%   export data to text
if export==1,
    D = [0,D];
    fn = fopen('c:\data\F.txt','w');
	for a = 1:length(F),
		i = ind(a);
		j = F(a);
        k = D(a);
		l = T(a);
		fprintf(fn, '%15.10f\t%15.10f\t%15.10f\t%15.10f\n', i, j, k, l);
   end
   fclose(fn);
end

